@@ -246,7 +246,7 @@ check_new_slave_physical_port (CEPageMaster *self, NMConnection *conn)
 		if (dev == dev2) {
 			nm_connection_editor_warning (CE_PAGE (self)->parent_window,
 			                              _("Duplicate slaves"),
-			                              _("Slaves '%s' and '%s' both apply to device '%s'"),
+			                              _("Slaves “%s” and “%s” both apply to device “%s”"),
 			                              nm_connection_get_id (conn),
 			                              nm_connection_get_id (conn2),
 			                              nm_device_get_iface (dev));
@@ -257,8 +257,8 @@ check_new_slave_physical_port (CEPageMaster *self, NMConnection *conn)
 		if (self->aggregating && id && id2 && !strcmp (id, id2)) {
 			nm_connection_editor_warning (CE_PAGE (self)->parent_window,
 			                              _("Duplicate slaves"),
-			                              _("Slaves '%s' and '%s' apply to different virtual "
-			                                "ports ('%s' and '%s') of the same physical device."),
+			                              _("Slaves “%s” and “%s” apply to different virtual "
+			                                "ports (“%s” and “%s”) of the same physical device."),
 			                              nm_connection_get_id (conn),
 			                              nm_connection_get_id (conn2),
 			                              nm_device_get_iface (dev),
@@ -357,7 +357,8 @@ add_response_cb (NMConnectionEditor *editor, GtkResponseType response, gpointer
 }
 
 static void
-add_connection (NMConnection *connection,
+add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
+                NMConnection *connection,
                 gpointer user_data)
 {
 	CEPageMaster *self = user_data;
@@ -398,10 +399,8 @@ add_connection (NMConnection *connection,
 	editor = nm_connection_editor_new (priv->toplevel,
 	                                   connection,
 	                                   CE_PAGE (self)->client);
-	if (!editor) {
-		g_object_unref (connection);
+	if (!editor)
 		return;
-	}
 
 	g_signal_connect (editor, "done", G_CALLBACK (add_response_cb), self);
 	nm_connection_editor_run (editor);
@@ -484,7 +483,8 @@ connection_double_clicked_cb (GtkTreeView *tree_view,
 }
 
 static void
-delete_result_cb (NMRemoteConnection *connection,
+delete_result_cb (FUNC_TAG_DELETE_CONNECTION_RESULT_IMPL,
+                  NMRemoteConnection *connection,
                   gboolean deleted,
                   gpointer user_data)
 {
