@@ -97,92 +97,6 @@ vpn_get_plugin_infos (void)
 	return plugins;
 }
 
-typedef struct {
-	VpnImportSuccessCallback callback;
-	gpointer user_data;
-} ActionInfo;
-
-static void
-import_vpn_from_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
-{
-	char *filename = NULL;
-	ActionInfo *info = (ActionInfo *) user_data;
-	GSList *iter;
-	NMConnection *connection = NULL;
-	GError *error = NULL;
-
-	if (response != GTK_RESPONSE_ACCEPT)
-		goto out;
-
-	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-	if (!filename) {
-		g_warning ("%s: didn't get a filename back from the chooser!", __func__);
-		goto out;
-	}
-
-	for (iter = vpn_get_plugin_infos (); !connection && iter; iter = iter->next) {
-		NMVpnEditorPlugin *plugin;
-
-		plugin = nm_vpn_plugin_info_get_editor_plugin (iter->data);
-		g_clear_error (&error);
-		connection = nm_vpn_editor_plugin_import (plugin, filename, &error);
-	}
-
-	if (connection)
-		info->callback (connection, info->user_data);
-	else {
-		GtkWidget *err_dialog;
-		char *bname = g_path_get_basename (filename);
-
-		err_dialog = gtk_message_dialog_new (NULL,
-		                                     GTK_DIALOG_DESTROY_WITH_PARENT,
-		                                     GTK_MESSAGE_ERROR,
-		                                     GTK_BUTTONS_OK,
-		                                     _("Cannot import VPN connection"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
-		                                 _("The file '%s' could not be read or does not contain recognized VPN connection information\n\nError: %s."),
-		                                 bname, error ? error->message : _("unknown error"));
-		g_free (bname);
-		g_signal_connect (err_dialog, "delete-event", G_CALLBACK (gtk_widget_destroy), NULL);
-		g_signal_connect (err_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-		gtk_widget_show_all (err_dialog);
-		gtk_window_present (GTK_WINDOW (err_dialog));
-	}
-
-	g_clear_error (&error);
-	g_free (filename);
-
-out:
-	gtk_widget_hide (dialog);
-	gtk_widget_destroy (dialog);
-	g_free (info);
-}
-
-void
-vpn_import (VpnImportSuccessCallback callback, gpointer user_data)
-{
-	GtkWidget *dialog;
-	ActionInfo *info;
-	const char *home_folder;
-
-	dialog = gtk_file_chooser_dialog_new (_("Select file to import"),
-	                                      NULL,
-	                                      GTK_FILE_CHOOSER_ACTION_OPEN,
-	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-	                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-	                                      NULL);
-	home_folder = g_get_home_dir ();
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), home_folder);
-
-	info = g_malloc0 (sizeof (ActionInfo));
-	info->callback = callback;
-	info->user_data = user_data;
-
-	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (import_vpn_from_file_cb), info);
-	gtk_widget_show_all (dialog);
-	gtk_window_present (GTK_WINDOW (dialog));
-}
-
 static void
 export_vpn_to_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 {
@@ -215,7 +129,7 @@ export_vpn_to_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 		                                         GTK_DIALOG_DESTROY_WITH_PARENT,
 		                                         GTK_MESSAGE_QUESTION,
 		                                         GTK_BUTTONS_CANCEL,
-		                                         _("A file named \"%s\" already exists."),
+		                                         _("A file named “%s” already exists."),
 		                                         bname);
 		gtk_dialog_add_buttons (GTK_DIALOG (replace_dialog), _("_Replace"), GTK_RESPONSE_OK, NULL);
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (replace_dialog),
@@ -257,7 +171,7 @@ done:
 		                                     GTK_BUTTONS_OK,
 		                                     _("Cannot export VPN connection"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
-		                                 _("The VPN connection '%s' could not be exported to %s.\n\nError: %s."),
+		                                 _("The VPN connection “%s” could not be exported to %s.\n\nError: %s."),
 		                                 id ? id : "(unknown)", bname, error ? error->message : "unknown error");
 		g_free (bname);
 		g_signal_connect (err_dialog, "delete-event", G_CALLBACK (gtk_widget_destroy), NULL);
@@ -292,7 +206,7 @@ vpn_export (NMConnection *connection)
 		return;
 	}
 
-	dialog = gtk_file_chooser_dialog_new (_("Export VPN connection..."),
+	dialog = gtk_file_chooser_dialog_new (_("Export VPN connection…"),
 	                                      NULL,
 	                                      GTK_FILE_CHOOSER_ACTION_SAVE,
 	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
