@@ -146,7 +146,7 @@ wireless_security_init (gsize obj_size,
                         WSFillConnectionFunc fill_connection,
                         WSUpdateSecretsFunc update_secrets,
                         WSDestroyFunc destroy,
-                        const char *ui_file,
+                        const char *ui_resource,
                         const char *ui_widget_name,
                         const char *default_field)
 {
@@ -154,7 +154,7 @@ wireless_security_init (gsize obj_size,
 	GError *error = NULL;
 
 	g_return_val_if_fail (obj_size > 0, NULL);
-	g_return_val_if_fail (ui_file != NULL, NULL);
+	g_return_val_if_fail (ui_resource != NULL, NULL);
 	g_return_val_if_fail (ui_widget_name != NULL, NULL);
 
 	sec = g_slice_alloc0 (obj_size);
@@ -170,9 +170,9 @@ wireless_security_init (gsize obj_size,
 	sec->default_field = default_field;
 
 	sec->builder = gtk_builder_new ();
-	if (!gtk_builder_add_from_file (sec->builder, ui_file, &error)) {
-		g_warning ("Couldn't load UI builder file %s: %s",
-		           ui_file, error->message);
+	if (!gtk_builder_add_from_resource (sec->builder, ui_resource, &error)) {
+		g_warning ("Couldn't load UI builder resource %s: %s",
+		           ui_resource, error->message);
 		g_error_free (error);
 		wireless_security_unref (sec);
 		return NULL;
@@ -181,7 +181,7 @@ wireless_security_init (gsize obj_size,
 	sec->ui_widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, ui_widget_name));
 	if (!sec->ui_widget) {
 		g_warning ("Couldn't load UI widget '%s' from UI file %s",
-		           ui_widget_name, ui_file);
+		           ui_widget_name, ui_resource);
 		wireless_security_unref (sec);
 		return NULL;
 	}
